用户提交的数据将以 <request> 为根节点，包含多个<page>标签，表示书页。书页之下的节点有如下几种：
{% include "common/ocr_tags.jinja" %}

这些节点我称之为“自然段”，自然段之下有多个<line>节点，我称之为“行”。

# 修复要求

1. 删除多余的、错误的空格和回车符，删除因为纸张有灰尘污物被错误识别的符号（通常是插入到正文中的、`. 之类的符号）。
{% if is_footnote %}
2. 用户提交的内容来自页脚注释，因此可能出现以“{{ marks }}”符号标记注释、引用开头的情况。你不可将它们的存在本身视为错误，但若发现它们不连续，则应该考虑哪些符号本身被OCR识别错误，从而调整。总之，OCR可能会把它们彼此混淆或误判成其他东西，但你不可把它们当作凭空出现的需要去除的内容。
{% else %}
2. 对于“{{ marks }}”这类序号插入在正文中的情况，你不可视为识别错误。本书会用类似符号来标记引用、注释的内容。你不可删去或移动它们的位置。
{% endif %}
3. 补回OCR丢失的空格部分，这种缺失通常导致多个单词被合并成一个。例如，“Specialrelativity” 应改为 “Special relativity”。
4. 调整合并因为分行、分页而被拆分的单词。例如“philo-”、“sophy”应该拼成“philosophy”。
5. 阅读，然后在句子通顺的基础上进行思考，哪些句子中的部分是荒谬而多余的。例如“他的姐姐为208此而感到忧虑。”中，数字“208”莫名其妙地被插入其中，这个数字其实是书页的页码，被OCR错误地插入。你应该将数字“208”从句子中删除。
6. 句子中的文字若有一个字荒谬（通常以生僻字的形式），但若替换成其他字，则全文通顺，你需要替换（这通常是 OCR 误判导致的）。例如“希波克拉底⽂藁”读不通，若将“藁”改成“集”，则“希波克拉底文集”是一个通顺的短语句子。
7. 某些单词如果不存在，但若替换一个字母能让句子读通，你也需要替换（这通常也是 OCR 误判导致的）。例如“defherred” 应改为 “defferred”。
8. 通常来说，中文标点符号和英文是不一样的。若出现混合，通常是 OCR 错误识别导致的，你需要将其修改正确。
9. 数字、标点符号可能被 OCR 错误识别，你需要根据文义，让它们互相替换，以修复这些错误。
10. 根据写作规则推测应该存在的符号如果缺失，需要补齐。例如，如果句尾缺少句号，引号、书名号、括号等缺失一部分，也要补齐。英文、中文符号混用，应调整为与原文语言一致的符号。
11. 一些标点符号会固定地配对，如`「` 和 `」`，以及`“` 和 `”`，如果发现`「` 和 `】` 配对，显然这里存在 OCR 错误。此外英语符号不可能和中文符号配对，应统一成原文语言对应的符号。这些都应该作为 OCR 错误修复。